@@ -1,20 +1,20 @@
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA  02110-1301, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #include "config.h"
@@ -33,11 +33,8 @@
 #include <glib/gstdio.h>
 
 #include <libtracker-common/tracker-common.h>
-
 #include <libtracker-db/tracker-db.h>
-
-#include <libtracker-miner/tracker-miner-manager.h>
-#include <libtracker-miner/tracker-crawler.h>
+#include <libtracker-miner/tracker-miner.h>
 
 #include "tracker-miner-files-reindex-client.h"
 
@@ -51,8 +48,19 @@
 	"\n" \
 	"  http://www.gnu.org/licenses/gpl.txt\n"
 
-static gboolean should_kill;
-static gboolean should_terminate;
+#define OPTION_TERM_ALL "all"
+#define OPTION_TERM_STORE "store"
+#define OPTION_TERM_MINERS "miners"
+
+typedef enum {
+	TERM_NONE,
+	TERM_ALL,
+	TERM_STORE,
+	TERM_MINERS
+} TermOption;
+
+static TermOption kill_option = TERM_NONE;
+static TermOption terminate_option = TERM_NONE;
 static gboolean hard_reset;
 static gboolean soft_reset;
 static gboolean remove_config;
@@ -60,14 +68,18 @@ static gboolean start;
 static const gchar **reindex_mime_types;
 static gboolean print_version;
 
+static gboolean term_option_arg_func (const gchar  *option_value,
+                                      const gchar  *value,
+                                      gpointer      data,
+                                      GError      **error);
+
 static GOptionEntry entries[] = {
-	{ "kill", 'k', 0, G_OPTION_ARG_NONE, &should_kill,
-	  N_("Use SIGKILL to stop all tracker processes found - guarantees death :)"),
-	  NULL },
-	{ "terminate", 't', 0, G_OPTION_ARG_NONE, &should_terminate,
-	  N_("Use SIGTERM to stop all tracker processes found"),
-	  NULL
-	},
+	{ "kill", 'k', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, term_option_arg_func,
+	  N_("Use SIGKILL to stop all matching processes, either \"store\", \"miners\" or \"all\" may be used, no parameter equals \"all\""),
+	  N_("APPS") },
+	{ "terminate", 't', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, term_option_arg_func,
+	  N_("Use SIGTERM to stop all matching processes, either \"store\", \"miners\" or \"all\" may be used, no parameter equals \"all\""),
+	  N_("APPS") },
 	{ "hard-reset", 'r', 0, G_OPTION_ARG_NONE, &hard_reset,
 	  N_("Kill all Tracker processes and remove all databases"),
 	  NULL },
@@ -193,6 +205,41 @@ crawler_finished_cb (TrackerCrawler *crawler,
 	g_main_loop_quit (user_data);
 }
 
+static gboolean
+term_option_arg_func (const gchar  *option_value,
+                      const gchar  *value,
+                      gpointer      data,
+                      GError      **error)
+{
+	TermOption option;
+
+	if (!value) {
+		value = OPTION_TERM_ALL;
+	}
+
+	if (strcmp (value, OPTION_TERM_ALL) == 0) {
+		option = TERM_ALL;
+	} else if (strcmp (value, OPTION_TERM_STORE) == 0) {
+		option = TERM_STORE;
+	} else if (strcmp (value, OPTION_TERM_MINERS) == 0) {
+		option = TERM_MINERS;
+	} else {
+		g_set_error_literal (error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED,
+		                     "Only one of 'all', 'store' and 'miners' are allowed");
+		return FALSE;
+	}
+
+	if (strcmp (option_value, "-k") == 0 ||
+	    strcmp (option_value, "--kill") == 0) {
+		kill_option = option;
+	} else if (strcmp (option_value, "-t") == 0 ||
+	           strcmp (option_value, "--terminate") == 0) {
+		terminate_option = option;
+	}
+
+	return TRUE;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -226,11 +273,11 @@ main (int argc, char **argv)
 		return EXIT_SUCCESS;
 	}
 
-	if (should_kill && should_terminate) {
+	if (kill_option != TERM_NONE && terminate_option != TERM_NONE) {
 		g_printerr ("%s\n",
 		            _("You can not use the --kill and --terminate arguments together"));
 		return EXIT_FAILURE;
-	} else if ((hard_reset || soft_reset) && should_terminate) {
+	} else if ((hard_reset || soft_reset) && terminate_option != TERM_NONE) {
 		g_printerr ("%s\n",
 		            _("You can not use the --terminate with --hard-reset or --soft-reset, --kill is implied"));
 		return EXIT_FAILURE;
@@ -242,13 +289,14 @@ main (int argc, char **argv)
 
 	if (hard_reset || soft_reset) {
 		/* Imply --kill */
-		should_kill = TRUE;
+		kill_option = TERM_ALL;
 	}
 
 	/* Unless we are stopping processes or listing processes,
 	 * don't iterate them.
 	 */
-	if (should_kill || should_terminate ||
+	if (kill_option != TERM_NONE ||
+	    terminate_option != TERM_NONE ||
 	    (!start && !remove_config)) {
 		pids = get_pids ();
 		str = g_strdup_printf (g_dngettext (NULL,
@@ -283,7 +331,9 @@ main (int argc, char **argv)
 				gchar *basename;
 
 				basename = g_path_get_basename (strv[0]);
-				if (g_str_has_prefix (basename, "tracker") == TRUE &&
+
+				if ((g_str_has_prefix (basename, "tracker") == TRUE ||
+				     g_str_has_prefix (basename, "lt-tracker") == TRUE) &&
 				    g_str_has_suffix (basename, "-control") == FALSE) {
 					pid_t pid;
 
@@ -292,7 +342,14 @@ main (int argc, char **argv)
 					g_print ("%s\n", str);
 					g_free (str);
 
-					if (should_terminate) {
+					if (terminate_option != TERM_NONE) {
+						if ((terminate_option == TERM_STORE &&
+						     !g_str_has_suffix (basename, "tracker-store")) ||
+						    (terminate_option == TERM_MINERS &&
+						     !strstr (basename, "tracker-miner"))) {
+							continue;
+						}
+
 						if (kill (pid, SIGTERM) == -1) {
 							const gchar *errstr = g_strerror (errno);
 
@@ -306,7 +363,14 @@ main (int argc, char **argv)
 							g_print ("  %s\n", str);
 							g_free (str);
 						}
-					} else if (should_kill) {
+					} else if (kill_option != TERM_NONE) {
+						if ((kill_option == TERM_STORE &&
+						     !g_str_has_suffix (basename, "tracker-store")) ||
+						    (kill_option == TERM_MINERS &&
+						     !strstr (basename, "tracker-miner"))) {
+							continue;
+						}
+
 						if (kill (pid, SIGKILL) == -1) {
 							const gchar *errstr = g_strerror (errno);
 
@@ -437,15 +501,12 @@ main (int argc, char **argv)
 			display_name = tracker_miner_manager_get_display_name (manager, l->data);
 
 			if (!tracker_miner_manager_get_status (manager, l->data, NULL, &progress)) {
-				g_printerr ("  %s: %s (%s)\n",
-				            _("Failed"),
+				g_printerr ("  ✗ %s (%s)\n",
 				            display_name,
-				            _("Could not get miner status"));
+				            _("perhaps a disabled plugin?"));
 			} else {
-				g_print ("  %s: %s (%3.0f%%)\n",
-				         _("Done"),
-				         display_name,
-				         progress * 100);
+				g_print ("  ✓ %s\n",
+				         display_name);
 			}
 
 			g_free (l->data);
