@@ -251,42 +251,6 @@ read_outline (PDFDoc               *document,
 	}
 }
 
-static void
-insert_keywords (TrackerSparqlBuilder *metadata,
-                 gchar                *keywords)
-{
-	char *saveptr, *p;
-	size_t len;
-
-	p = keywords;
-	keywords = strchr (keywords, '"');
-
-	if (keywords) {
-		keywords++;
-	} else {
-		keywords = p;
-	}
-
-	len = strlen (keywords);
-	if (keywords[len - 1] == '"') {
-		keywords[len - 1] = '\0';
-	}
-
-	for (p = strtok_r (keywords, ",;", &saveptr);
-	     p;
-	     p = strtok_r (NULL, ",;", &saveptr)) {
-		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
-
-		tracker_sparql_builder_object_blank_open (metadata);
-		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "nao:Tag");
-
-		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
-		tracker_sparql_builder_object_unvalidated (metadata, p);
-
-		tracker_sparql_builder_object_blank_close (metadata);
-	}
-}
 
 static void
 page_get_size (Page    *page,
@@ -321,6 +285,7 @@ extract_content (PDFDoc *document,
 	GString *string;
 	gint n_pages, i;
 	gsize n_bytes_remaining;
+	GTimer *timer;
 
 	n_pages = document->getNumPages();
 	string = g_string_new ("");
@@ -328,7 +293,9 @@ extract_content (PDFDoc *document,
 	n_bytes_remaining = n_bytes;
 	catalog = document->getCatalog();
 
-	while (i < n_pages && n_bytes_remaining > 0) {
+	timer = g_timer_new ();
+
+	while (i < n_pages && n_bytes_remaining > 0 && g_timer_elapsed (timer, NULL) < 5) {
 		Gfx *gfx;
 		GooString *sel_text;
 		TextOutputDev *text_dev;
@@ -376,14 +343,18 @@ extract_content (PDFDoc *document,
 
 		delete gfx;
 		delete text_dev;
+		delete sel_text;
 	}
 
+	g_timer_destroy (timer);
+
 	return g_string_free (string, FALSE);
 }
 
 static void
 write_pdf_data (PDFData               data,
-                TrackerSparqlBuilder *metadata)
+                TrackerSparqlBuilder *metadata,
+                GPtrArray            *keywords)
 {
 	if (!tracker_is_empty_string (data.title)) {
 		tracker_sparql_builder_predicate (metadata, "nie:title");
@@ -411,7 +382,7 @@ write_pdf_data (PDFData               data,
 	}
 
 	if (!tracker_is_empty_string (data.keywords)) {
-		insert_keywords (metadata, data.keywords);
+		tracker_keywords_parse (keywords, data.keywords);
 	}
 }
 
@@ -510,6 +481,8 @@ extract_pdf (const gchar          *uri,
 	gsize n_bytes;
 	Object obj;
 	Catalog *catalog;
+	GPtrArray *keywords;
+	guint i;
 
 	g_type_init ();
 
@@ -568,6 +541,7 @@ extract_pdf (const gchar          *uri,
 	}
 	obj.free ();
 
+	keywords = g_ptr_array_new ();
 
 	catalog = document->getCatalog ();
 	if (catalog && catalog->isOk ()) {
@@ -590,14 +564,14 @@ extract_pdf (const gchar          *uri,
 			md.date = (gchar *) tracker_coalesce_strip (3, pd.creation_date, xd->date, xd->time_original);
 			md.author = (gchar *) tracker_coalesce_strip (2, pd.author, xd->creator);
 
-			write_pdf_data (md, metadata);
+			write_pdf_data (md, metadata, keywords);
 
 			if (xd->keywords) {
-				insert_keywords (metadata, xd->keywords);
+				tracker_keywords_parse (keywords, xd->keywords);
 			}
 
 			if (xd->pdf_keywords) {
-				insert_keywords (metadata, xd->pdf_keywords);
+				tracker_keywords_parse (keywords, xd->pdf_keywords);
 			}
 
 			if (xd->publisher) {
@@ -660,7 +634,7 @@ extract_pdf (const gchar          *uri,
 					camera = g_strdup (xd->model);
 				}
 
-				tracker_sparql_builder_predicate (metadata, "nmm:camera");
+				tracker_sparql_builder_predicate (metadata, "nfo:device");
 				tracker_sparql_builder_object_unvalidated (metadata, camera);
 				g_free (camera);
 			}
@@ -787,9 +761,33 @@ extract_pdf (const gchar          *uri,
 		/* So if we are here we have NO XMP data and we just
 		 * write what we know from Poppler.
 		 */
-		write_pdf_data (pd, metadata);
+		write_pdf_data (pd, metadata, keywords);
+
+		g_free (pd.keywords);
+		g_free (pd.title);
+		g_free (pd.subject);
+		g_free (pd.creation_date);
+		g_free (pd.author);
+		g_free (pd.date);
 	}
 
+	for (i = 0; i < keywords->len; i++) {
+		gchar *p;
+
+		p = (gchar *) g_ptr_array_index (keywords, i);
+
+		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
+
+		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nao:Tag");
+		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
+		tracker_sparql_builder_object_unvalidated (metadata, p);
+		tracker_sparql_builder_object_blank_close (metadata);
+		g_free (p);
+	}
+	g_ptr_array_free (keywords, TRUE);
+
 	tracker_sparql_builder_predicate (metadata, "nfo:pageCount");
 	tracker_sparql_builder_object_int64 (metadata, document->getNumPages());
 
